#!/usr/bin/perl


#  Copyright notice:  Copyright © 2001 The Regents of the University of
#  California.  All rights reserved.
#
#  Other notices:  This software and text  files are provided as an
#  educational resource for non-commercial use only.
#
#  1.  Redistributions of this software must retain the copyright notice
#  on this page, this list of conditions and the following disclaimer.
#
#  2.Software, computer files, books or other documentation that incorporates
#  all or part of this file must reproduce the copyright notice on this page,
#  this list of conditions and the following disclaimer in the software
#  and/or other materials provided with the distribution.
#
#  THIS SOFTWARE IS PROVIDED BY THE REGENTS OF THE UNIVERSITY OF CALIFORNIA
#  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
#  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
#  CONTRIBUTORS TO THE WORK BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS, LOSS OR USE, DATA OR PROFITS, OR
#  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE), ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
#  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


package shuffle;

@ISA = ('Exporter');
@EXPORT = qw(&shuffle_quiz &make_dtd);


#use strict;
use XML::LibXML;

my @answer_key = ();
my $problem_count = 0;
my $answer_key = '';
my $solution_on = 1;
$dtd;


sub make_dtd{

   return if ($dtd);

   my @dtd_string = <DATA>;

   my $dtd_string = join('',@dtd_string);

   $dtd = XML::LibXML::Dtd->parse_string($dtd_string);

}

sub shuffle_quiz {
   my $doc_string = shift;
   $solution_on = shift;
   $no_shuffling = shift;

   @answer_key = ();
   $problem_count = 0;
   $answer_key = '';

   my $doc = XML::LibXML->new->parse_string($doc_string);

   $doc->validate($dtd);

   my $root = $doc->getDocumentElement;

   my $header = $root->findvalue('/quiz/header');
   my $footer = $root->findvalue('/quiz/footer');

   my @keep_prob  = $root->findnodes('problem | keep');

   shuffle(\@keep_prob);

   my $out = $header;

   foreach my $keep_prob (@keep_prob) {
      my $node_name =  $keep_prob->getLocalName(), "\n";
      #print "$node_name \n";
      if ($node_name eq 'keep') {
	 #print "entering keep\n";
	 foreach my $prob ($keep_prob->findnodes('problem')) {
	    #print  $prob->getLocalName(), "\n";
	    $out .= problem_to_string($prob);
	 }
	 #print "exiting keep\n";
      }
      else {
	 $out .=  problem_to_string($keep_prob);
      }
   }

#   print $out;

#answer key:
for(my $i=0; $i < scalar(@answer_key); $i++) {
   my $correct = $answer_key[$i];
   if ($correct) {
      #print "problem $i: $correct\n\n" 
      $answer_key .=  "problem $i: $correct\n\n";
   }
}
$out .= $footer;
return ($out,$answer_key);
}

sub problem_to_string {
   my $node = shift;
   $problem_count++;
   my $problem_string = "\n".'\filbreak'."\n $problem_count .";
   $problem_string .= $node->findvalue('question');
   my @choices = $node->findnodes('choice');
   if (@choices) {
      $problem_string .= choices_to_string(@choices);
   }
   if ($solution_on == 1) {
      my $solution = $node->findvalue('solution');
      $problem_string .= $solution if ($solution);
   }
   return $problem_string;
}

sub shuffle {
    return if ($no_shuffling == 1);
    my $arr = shift;
    for (my $i = @$arr; -- $i;) {
        my $ran = int rand ($i + 1);
       ($arr->[$i], $arr->[$ran]) = ($arr->[$ran], $arr->[$i]);
    }
}

sub choices_to_string {
      my @choices = @_;
      my $i=0;
      my $j=0;
      my $n = scalar(@choices);
      my $n_a = 97;# integer corresponding to "a";
      die "too many items: $n > 26\n" if ($n > 26);
      my $choice_string = '\begin{description}' . "\n";

      my (@choice_index,@anywhere , @shuffle_choice);
      foreach my $choice (@choices) {
	 my $fix =  $choice->findvalue('@fix');
	 push  @anywhere, $j if ($fix ne 'here');
	 $choice_index[$j] = $j;
	 $j++;
      }
      if (scalar(@anywhere) > 0) {
      my @shuffle_anywhere = @anywhere;
      shuffle (\@shuffle_anywhere);

      for($i=0;$i < scalar(@anywhere);$i++){
	 $choice_index[$anywhere[$i]] = $shuffle_anywhere[$i];
      }
   }

      for($i=0;$i < scalar(@choices);$i++){
	 my $label = chr($n_a+$i);
	 $choice_string .=  '\item[(' . $label  . ')] ' .  $choices[$choice_index[$i]]->string_value() . "\n";
	 $answer_key[$problem_count] = $label if ($choices[$choice_index[$i]]->findvalue('@type') eq 'right');
      }
      $choice_string .= '\end{description}' . "\n";

      return $choice_string;
}

1;

__DATA__
<!ELEMENT quiz (header?,(img | keep | def | problem)+,footer?)>
<!ELEMENT img EMPTY>
<!ATTLIST img file CDATA #REQUIRED>
<!ATTLIST img width CDATA #IMPLIED>
<!ATTLIST img height CDATA #IMPLIED>
<!ATTLIST img border CDATA #IMPLIED>
<!ELEMENT problem  ((img | def | popup)*,question,(img | popup| choice | def)*,solution?)>
<!ELEMENT popup (img*,text,img*)>
<!ATTLIST popup name CDATA #IMPLIED>
<!ELEMENT text (#PCDATA)>
<!ELEMENT solution (#PCDATA)>
<!ELEMENT def (#PCDATA)>
<!ELEMENT keep (problem+)>
<!ELEMENT choice (img*,text,img*)>
<!ATTLIST choice type (right | wrong | also)  #IMPLIED>
<!ATTLIST choice fix (here | anywhere)  #IMPLIED>
<!ELEMENT question  (#PCDATA)>
<!ELEMENT header  (#PCDATA)>
<!ELEMENT footer  (#PCDATA)>
